
/** Framework for exercise 4.1 and 4.2
*
* @author
*   Gabriel Zachmann, Sep 2006
*
* @bug
*   I'm sure there is still some bugs left ....
*
*
**/

#include <QtGui>
#include <QtOpenGL>
#include <QMouseEvent>

#include <math.h>

#include "glwidget.h"

const GLdouble GLWidget::FRONT     =   10.0;
const GLdouble GLWidget::BACK      =  400.0;
const GLdouble GLWidget::SV_TRANSZ =    -50;
const GLdouble GLWidget::PV_TRANSZ =   -300;

GLWidget::GLWidget(QWidget * father )
: QGLWidget(father)
, m_singleView(true)
, m_xTrans(0.0)
, m_yTrans(0.0)
, m_zTrans(SV_TRANSZ)
{
	m_backgroundcolor[0] = 0.0;
	m_backgroundcolor[1] = 0.0;
	m_backgroundcolor[2] = 0.0;
	resetRotation();
    setFocusPolicy( Qt::ClickFocus ); //needed for keyboard-input
	genCubeCoords();

	srand( (unsigned)time( NULL ) );
	genPolygonCoords();
}


GLWidget::~GLWidget()
{
    makeCurrent();
}

// set rotation Matrix for trackball to Identity
void GLWidget::resetRotation()
{
	for(int i = 1; i < 4; i++)
	{
		for(int j = 0; j < i; j++)
		{
			m_RotationMatrix[i*4+j] = m_RotationMatrix[j*4+i]= 0.0;
		}
	}
	m_RotationMatrix[0] = m_RotationMatrix[5] = m_RotationMatrix[10] = m_RotationMatrix[15] = 1.0;
}

QSize GLWidget::minimumSizeHint() const
{
    return QSize(800, 800);
}

QSize GLWidget::sizeHint() const
{
    return QSize(800, 800);
}


void GLWidget::setBackgroundColor(float v)
{
	m_backgroundcolor[0] = v;
    m_backgroundcolor[1] = v;
    m_backgroundcolor[2] = v;
	updateGL();
}


void GLWidget::initializeGL()
{
	// set color buffer to this color if glClear() is called
    glClearColor( m_backgroundcolor[0], m_backgroundcolor[1], m_backgroundcolor[2], 0.0 );
	// set openGL states
    glEnable(GL_DEPTH_TEST);
	glDisable(GL_CULL_FACE);
	glShadeModel(GL_FLAT); // we use flat shading
	glEnable(GL_LIGHTING); // enable lighting
	glEnable(GL_LIGHT0);
	glEnable(GL_COLOR_MATERIAL);
	glColorMaterial( GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE );
}


// Warning: don't make paintGL() and resizeGL() const!
// otherwise, the rendering will produce complete garbage.
// (is this a bug in Qt? g++? opengl?)

void GLWidget::paintGL()
{
	// clear framebuffer
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	// push current matrix to stack
	glPushMatrix();

	// manage simple navigation ( rotation and translation )
	glLoadIdentity();
	glTranslatef(m_xTrans, m_yTrans, m_zTrans);
	glMultMatrixf(m_RotationMatrix); //

	if ( m_singleView ) // for exercise one
	{
		glColor3f( 1.0f, 1.0f, 1.0f );
		Vertex cubeCenter( 0.0f, 0.0f, 0.0f );
		drawCube(cubeCenter);
	}
	else // exercise two
	{

		rasterPolygon(m_polygon);
	}

	// restore matrix from stack
	glPopMatrix();
}


void GLWidget::resizeGL( int w, int h )		// = width & height
{
    int side = qMin(w, h);
    glViewport((w - side) / 2, (h - side) / 2, side, side);

	updateProjectonMatrix(FRONT, BACK);
}

void GLWidget::updateProjectonMatrix(GLdouble front, GLdouble back)
{
	glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
	glFrustum(-1.0, 1.0, -1.0, 1.0, front, back); // width,height = 2, depth = back-front
    glMatrixMode(GL_MODELVIEW);
}

void GLWidget::mousePressEvent( QMouseEvent * e /*the event*/ )
{
    m_lastPos = e->pos();
}


void GLWidget::keyPressEvent ( QKeyEvent * e )
{
    if ( !isValid() )
        return;

    switch ( e->key() )
    {
        case Qt::Key_B: // switch background color if key "B" is pressed
            if ( m_backgroundcolor[0] == 1.0 )
			{
                setBackgroundColor( 0.0 );
			} else
			{
                setBackgroundColor( 1.0 );
			}
			glClearColor( m_backgroundcolor[0], m_backgroundcolor[1], m_backgroundcolor[2], 0.0 );
			updateGL();	// openGL window has to be redrawn
            break;
        default:
            return;
    }       
}


void GLWidget::mouseMoveEvent( QMouseEvent * e /*the event*/ )
{
    int dx = e->x() - m_lastPos.x();
    int dy = e->y() - m_lastPos.y();

	bool ctrl_key = e->modifiers() & Qt::MetaModifier;		// only needed for Mac OS X, but doesn't hurt on other OSes

	if ( (e->buttons() & Qt::RightButton) || ctrl_key ) // translation along z-Axis
	{
		m_zTrans += 0.2 * dy;
	}
	else if ( e->buttons() & Qt::MidButton ) // translation in xy-Plane
	{
		m_xTrans += 0.01 * dx;
		m_yTrans -= 0.01 * dy;
	}
	else if (e->buttons() & Qt::LeftButton)	// rotation
	{
		// openGL multiplies new transformations on the right
		// we want to apply rotation on the left
		glPushMatrix(); // push current openGL transform matrix to stack
		glLoadIdentity();
		glRotatef(0.5*dx,0.0, 1.0, 0.0);
		glRotatef(0.5*dy,1.0, 0.0, 0.0);
		glMultMatrixf(m_RotationMatrix);
		glGetFloatv(GL_MODELVIEW_MATRIX,m_RotationMatrix);
		glPopMatrix();
    }
    m_lastPos = e->pos();
	e->accept();
	updateGL();
}

void GLWidget::setSingleViewFrustum(bool checked)
{
	if ( checked )
	{
		updateProjectonMatrix(FRONT, BACK);
		m_xTrans = m_yTrans = 0;
		m_zTrans = SV_TRANSZ;
		m_singleView = true;
		resetRotation();
		updateGL();
	}
}

void GLWidget::setPolygonViewFrustum(bool checked)
{
	if ( checked )
	{
		updateProjectonMatrix(FRONT, BACK);
		m_xTrans = m_yTrans = 0;
		m_zTrans = PV_TRANSZ;
		resetRotation();
		m_singleView = false;
		updateGL();
	}
}

// ** TODO **
// rasterize a polygon and draw pixels as cubes
// INPUTS 
// polygon : 3D-vertices of Polygon
void GLWidget::rasterPolygon( const std::vector<Vertex> &polygon )
{
	glBegin(GL_LINE_LOOP);
	{
		for(unsigned int i = 0; i < polygon.size(); i++)
		{
			glVertex3f(polygon[i].x,polygon[i].y,polygon[i].z);
		}
	}
	glEnd();

	// draw polygon vertices
	for(unsigned int i = 0; i < polygon.size(); i++)
		drawCube(polygon[i]); // each vertex of polygon


}

// draw cube with center x,y,z and lengh 0.9
void GLWidget::drawCube(const Vertex &center)
{
	glPushMatrix();
	
	glTranslatef(center.x,center.y,center.z);
	glScalef( 0.45f, 0.45f ,0.45f );
	
	// draw unit cube at origin
	glBegin(GL_QUAD_STRIP);
	glNormal3f(m_cubeNormals[0].x,m_cubeNormals[0].y,m_cubeNormals[0].z);
	for( int i = 0; i < 4; i++)
		glVertex3f(m_cube[i].x,m_cube[i].y,m_cube[i].z);
	
	
	for( int i = 1; i < 3; i++)
	{
		glNormal3f(m_cubeNormals[i].x,m_cubeNormals[i].y,m_cubeNormals[i].z);
		int vidx = 2*(i+1);
		glVertex3f(m_cube[vidx].x,m_cube[vidx].y,m_cube[vidx].z);
		glVertex3f(m_cube[vidx+1].x,m_cube[vidx+1].y,m_cube[vidx+1].z);
	}
	glEnd();

	glBegin(GL_QUAD_STRIP);
	glNormal3f(m_cubeNormals[3].x,m_cubeNormals[3].y,m_cubeNormals[3].z);
	for( int i = 8; i < 12; i++)
		glVertex3f(m_cube[i].x,m_cube[i].y,m_cube[i].z);


	for( int i = 1; i < 3; i++)
	{
		glNormal3f(m_cubeNormals[3+i].x,m_cubeNormals[3+i].y,m_cubeNormals[3+i].z);
		int vidx = 8+2*(i+1);
		glVertex3f(m_cube[vidx].x,m_cube[vidx].y,m_cube[vidx].z);
		glVertex3f(m_cube[vidx+1].x,m_cube[vidx+1].y,m_cube[vidx+1].z);
	}
	glEnd();
	
	glPopMatrix();
}


void GLWidget::genCubeCoords()
{
	// generate vertices
	Vertex v0(-1,-1, 1);
	Vertex v1( 1,-1, 1);
	Vertex v2(-1, 1, 1);
	Vertex v3( 1, 1, 1);
	Vertex v4(-1,-1,-1);
	Vertex v5( 1,-1,-1);
	Vertex v6(-1, 1,-1);
	Vertex v7( 1, 1,-1);

	m_cube.reserve(16);

	// first quad-strip
	m_cube.push_back(v2);
	m_cube.push_back(v3);
	m_cube.push_back(v0);
	m_cube.push_back(v1);
	m_cube.push_back(v4);
	m_cube.push_back(v5);
	m_cube.push_back(v6);
	m_cube.push_back(v7);

	// second quad-strip
	m_cube.push_back(v1);
	m_cube.push_back(v5);
	m_cube.push_back(v3);
	m_cube.push_back(v7);
	m_cube.push_back(v2);
	m_cube.push_back(v6);
	m_cube.push_back(v0);
	m_cube.push_back(v4);

	// generate normals
	m_cubeNormals.reserve(6);
	m_cubeNormals.push_back(Vertex(0,0,1));
	m_cubeNormals.push_back(Vertex(0,-1,0));
	m_cubeNormals.push_back(Vertex(0,0,-1));
	m_cubeNormals.push_back(Vertex(1,0,0));
	m_cubeNormals.push_back(Vertex(0,1,0));
	m_cubeNormals.push_back(Vertex(-1,0,0));
}

void GLWidget::genPolygonCoords()
{
	m_polygon.clear();

	m_polygon.reserve(8);

	for( int x = -20; x < 20; x+=10 )
		m_polygon.push_back(Vertex(getRandomNumber(x,x+10),getRandomNumber(-20,0),0));

	for( int x = 20; x > -20; x-=10 )
		m_polygon.push_back(Vertex(getRandomNumber(x-10,x),getRandomNumber(0,20),0));
}
