
#ifndef GLWIDGET_H
#define GLWIDGET_H

#include <QGLWidget>
#include <vector>
#include <stdlib.h>
#include <stdio.h>
#include <time.h>


struct Vertex
{
	Vertex() : x(0), y(0), z(0) {}
	Vertex(float xf, float yf, float zf) : x(xf), y(yf), z(zf) {}
	float x,y,z;
};

typedef std::vector<Vertex> VertexVec;

class GLWidget : public QGLWidget
{
	static const GLdouble FRONT;
	static const GLdouble BACK;
	static const GLdouble SV_TRANSZ;
	static const GLdouble PV_TRANSZ;

    Q_OBJECT

public:
    GLWidget(QWidget * father = 0);
    ~GLWidget();

    QSize minimumSizeHint() const;
    QSize sizeHint() const;

public slots: // slots ( connected in Window::Windwo() )
	void setSingleViewFrustum(bool checked);
	void setPolygonViewFrustum(bool checked);
	void slotRandomPoly() { genPolygonCoords(); updateGL(); }

protected:
    virtual void initializeGL();
    virtual void paintGL();
    virtual void resizeGL( int width, int height );
	virtual void mousePressEvent( QMouseEvent *event );
	virtual void mouseMoveEvent( QMouseEvent *event );
    virtual void keyPressEvent ( QKeyEvent * event );

    void setBackgroundColor(float v);

private:
	void rasterPolygon(const std::vector<Vertex> &polygon); // ** TODO **
	void drawCube(const Vertex &center);
	void genCubeCoords();
	void genPolygonCoords();

	// trackball helper functions
	void updateProjectonMatrix(GLdouble front, GLdouble back); // set viewing frustum
	void resetRotation(); // set rotation matrix to Identity
	int getRandomNumber( int range_min, int range_max) { return static_cast<int>( (double)rand() / (RAND_MAX + 1) * (range_max - range_min) + range_min ); }

	bool			m_singleView;								//!< MODE: if true show Cube else show Polygon

	// some variables for trackball
	float			m_RotationMatrix[16];						//!< global rotation Matrix
	float			m_xTrans, m_yTrans, m_zTrans;				//!< global translation
    QPoint			m_lastPos;
	float			m_backgroundcolor[3];						//!< background color of openGL window
	VertexVec		m_polygon;
	VertexVec		m_cube;
	VertexVec		m_cubeNormals;
};

#endif
