
#include <QtGui>

#include "glwidget.h"
#include "window.h"

QGroupBox *Window::createGroupBox()
{
	QGroupBox *groupBox = new QGroupBox(tr("View mode"));

	// add two widgets to be able to switch between cube and polygon
	m_singleCubeView = new QRadioButton(tr("&Single cube"));
	m_polygonView = new QRadioButton(tr("&Polygon"));
	m_regenPolygon = new QPushButton(tr("&Random Poly"));
	// per default, set cube to be visible
	m_singleCubeView->setChecked(true);

	// the widgets are vertically aligned
	QVBoxLayout *vbox = new QVBoxLayout;
	vbox->addWidget(m_singleCubeView);
	vbox->addWidget(m_polygonView);
	vbox->addWidget(m_regenPolygon);
	vbox->addStretch(1);
	groupBox->setLayout(vbox);

	return groupBox;
}


Window::Window()
{
    m_glWidget = new GLWidget;

    QHBoxLayout *mainLayout = new QHBoxLayout;
	// this widget contains our openGL window
    mainLayout->addWidget(m_glWidget);
	// and this one the radio buttons
	mainLayout->addWidget(createGroupBox());
    setLayout(mainLayout);

	// connect signals and slots
	// if we click a radio button, the glwidget class need to be notified to apply the changes
	connect(m_singleCubeView,SIGNAL(toggled(bool)),m_glWidget,SLOT(setSingleViewFrustum(bool)));
	connect(m_polygonView,SIGNAL(toggled(bool)),m_glWidget,SLOT(setPolygonViewFrustum(bool)));
	connect( m_regenPolygon, SIGNAL(clicked()), m_glWidget, SLOT(slotRandomPoly()) );

    setWindowTitle( "Pineda" );
}

